X500ATTR_OID = "urn:oid:2.5.4."
PKCS_9 = "urn:oid:1.2.840.113549.1.9.1."
UCL_DIR_PILOT = "urn:oid:0.9.2342.19200300.100.1."

MAP = {
    "identifier": "urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
    "fro": {
        X500ATTR_OID + "3": "first_name",  # cn
        X500ATTR_OID + "4": "last_name",  # sn
        PKCS_9 + "1": "email",
        UCL_DIR_PILOT + "1": "uid",
    },
    "to": {
        "first_name": X500ATTR_OID + "3",
        "last_name": X500ATTR_OID + "4",
        "email": PKCS_9 + "1",
        "uid": UCL_DIR_PILOT + "1",
    },
}



# Copyright (C) 2010-2012 Yaco Sistemas (http://www.yaco.es)
# Copyright (C) 2009 Lorenzo Gil Sanchez <lorenzo.gil.sanchez@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import path

from . import views

urlpatterns = [
    path("login/", views.LoginView.as_view(), name="saml2_login"),
    path("acs/", views.AssertionConsumerServiceView.as_view(), name="saml2_acs"),
    path("logout/", views.LogoutInitView.as_view(), name="saml2_logout"),
    path("ls/", views.LogoutView.as_view(), name="saml2_ls"),
    path("ls/post/", views.LogoutView.as_view(), name="saml2_ls_post"),
    path("metadata/", views.MetadataView.as_view(), name="saml2_metadata"),
]



__author__ = "rolandh"

EDUPERSON_OID = "urn:oid:1.3.6.1.4.1.5923.1.1.1."
X500ATTR_OID = "urn:oid:2.5.4."
NOREDUPERSON_OID = "urn:oid:1.3.6.1.4.1.2428.90.1."
NETSCAPE_LDAP = "urn:oid:2.16.840.1.113730.3.1."
UCL_DIR_PILOT = "urn:oid:0.9.2342.19200300.100.1."
PKCS_9 = "urn:oid:1.2.840.113549.1.9.1."
UMICH = "urn:oid:1.3.6.1.4.1.250.1.57."
SCHAC = "urn:oid:1.3.6.1.4.1.25178.1.2."

MAP = {
    "identifier": "urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
    "fro": {
        EDUPERSON_OID + "2": "eduPersonNickname",
        EDUPERSON_OID + "9": "eduPersonScopedAffiliation",
        EDUPERSON_OID + "11": "eduPersonAssurance",
        EDUPERSON_OID + "10": "eduPersonTargetedID",
        EDUPERSON_OID + "4": "eduPersonOrgUnitDN",
        NOREDUPERSON_OID + "6": "norEduOrgAcronym",
        NOREDUPERSON_OID + "7": "norEduOrgUniqueIdentifier",
        NOREDUPERSON_OID + "4": "norEduPersonLIN",
        EDUPERSON_OID + "1": "eduPersonAffiliation",
        NOREDUPERSON_OID + "2": "norEduOrgUnitUniqueNumber",
        NETSCAPE_LDAP + "40": "userSMIMECertificate",
        NOREDUPERSON_OID + "1": "norEduOrgUniqueNumber",
        NETSCAPE_LDAP + "241": "displayName",
        UCL_DIR_PILOT + "37": "associatedDomain",
        EDUPERSON_OID + "6": "eduPersonPrincipalName",
        NOREDUPERSON_OID + "8": "norEduOrgUnitUniqueIdentifier",
        NOREDUPERSON_OID + "9": "federationFeideSchemaVersion",
        X500ATTR_OID + "53": "deltaRevocationList",
        X500ATTR_OID + "52": "supportedAlgorithms",
        X500ATTR_OID + "51": "houseIdentifier",
        X500ATTR_OID + "50": "uniqueMember",
        X500ATTR_OID + "19": "physicalDeliveryOfficeName",
        X500ATTR_OID + "18": "postOfficeBox",
        X500ATTR_OID + "17": "postalCode",
        X500ATTR_OID + "16": "postalAddress",
        X500ATTR_OID + "15": "businessCategory",
        X500ATTR_OID + "14": "searchGuide",
        EDUPERSON_OID + "5": "eduPersonPrimaryAffiliation",
        X500ATTR_OID + "12": "title",
        X500ATTR_OID + "11": "ou",
        X500ATTR_OID + "10": "o",
        X500ATTR_OID + "37": "cACertificate",
        X500ATTR_OID + "36": "userCertificate",
        X500ATTR_OID + "31": "member",
        X500ATTR_OID + "30": "supportedApplicationContext",
        X500ATTR_OID + "33": "roleOccupant",
        X500ATTR_OID + "32": "owner",
        NETSCAPE_LDAP + "1": "carLicense",
        PKCS_9 + "1": "email",
        NETSCAPE_LDAP + "3": "employeeNumber",
        NETSCAPE_LDAP + "2": "departmentNumber",
        X500ATTR_OID + "39": "certificateRevocationList",
        X500ATTR_OID + "38": "authorityRevocationList",
        NETSCAPE_LDAP + "216": "userPKCS12",
        EDUPERSON_OID + "8": "eduPersonPrimaryOrgUnitDN",
        X500ATTR_OID + "9": "street",
        X500ATTR_OID + "8": "st",
        NETSCAPE_LDAP + "39": "preferredLanguage",
        EDUPERSON_OID + "7": "eduPersonEntitlement",
        X500ATTR_OID + "2": "knowledgeInformation",
        X500ATTR_OID + "7": "l",
        X500ATTR_OID + "6": "c",
        X500ATTR_OID + "5": "serialNumber",
        X500ATTR_OID + "4": "sn",
        X500ATTR_OID + "3": "cn",
        UCL_DIR_PILOT + "60": "jpegPhoto",
        X500ATTR_OID + "65": "pseudonym",
        NOREDUPERSON_OID + "5": "norEduPersonNIN",
        UCL_DIR_PILOT + "3": "mail",
        UCL_DIR_PILOT + "25": "dc",
        X500ATTR_OID + "40": "crossCertificatePair",
        X500ATTR_OID + "42": "givenName",
        X500ATTR_OID + "43": "initials",
        X500ATTR_OID + "44": "generationQualifier",
        X500ATTR_OID + "45": "x500UniqueIdentifier",
        X500ATTR_OID + "46": "dnQualifier",
        X500ATTR_OID + "47": "enhancedSearchGuide",
        X500ATTR_OID + "48": "protocolInformation",
        X500ATTR_OID + "54": "dmdName",
        NETSCAPE_LDAP + "4": "employeeType",
        X500ATTR_OID + "22": "teletexTerminalIdentifier",
        X500ATTR_OID + "23": "facsimileTelephoneNumber",
        X500ATTR_OID + "20": "telephoneNumber",
        X500ATTR_OID + "21": "telexNumber",
        X500ATTR_OID + "26": "registeredAddress",
        X500ATTR_OID + "27": "destinationIndicator",
        X500ATTR_OID + "24": "x121Address",
        X500ATTR_OID + "25": "internationaliSDNNumber",
        X500ATTR_OID + "28": "preferredDeliveryMethod",
        X500ATTR_OID + "29": "presentationAddress",
        EDUPERSON_OID + "3": "eduPersonOrgDN",
        NOREDUPERSON_OID + "3": "norEduPersonBirthDate",
        UMICH + "57": "labeledURI",
        UCL_DIR_PILOT + "1": "uid",
        SCHAC + "1": "schacMotherTongue",
        SCHAC + "2": "schacGender",
        SCHAC + "3": "schacDateOfBirth",
        SCHAC + "4": "schacPlaceOfBirth",
        SCHAC + "5": "schacCountryOfCitizenship",
        SCHAC + "6": "schacSn1",
        SCHAC + "7": "schacSn2",
        SCHAC + "8": "schacPersonalTitle",
        SCHAC + "9": "schacHomeOrganization",
        SCHAC + "10": "schacHomeOrganizationType",
        SCHAC + "11": "schacCountryOfResidence",
        SCHAC + "12": "schacUserPresenceID",
        SCHAC + "13": "schacPersonalPosition",
        SCHAC + "14": "schacPersonalUniqueCode",
        SCHAC + "15": "schacPersonalUniqueID",
        SCHAC + "17": "schacExpiryDate",
        SCHAC + "18": "schacUserPrivateAttribute",
        SCHAC + "19": "schacUserStatus",
        SCHAC + "20": "schacProjectMembership",
        SCHAC + "21": "schacProjectSpecificRole",
    },
    "to": {
        "roleOccupant": X500ATTR_OID + "33",
        "gn": X500ATTR_OID + "42",
        "norEduPersonNIN": NOREDUPERSON_OID + "5",
        "title": X500ATTR_OID + "12",
        "facsimileTelephoneNumber": X500ATTR_OID + "23",
        "mail": UCL_DIR_PILOT + "3",
        "postOfficeBox": X500ATTR_OID + "18",
        "fax": X500ATTR_OID + "23",
        "telephoneNumber": X500ATTR_OID + "20",
        "norEduPersonBirthDate": NOREDUPERSON_OID + "3",
        "rfc822Mailbox": UCL_DIR_PILOT + "3",
        "dc": UCL_DIR_PILOT + "25",
        "countryName": X500ATTR_OID + "6",
        "emailAddress": PKCS_9 + "1",
        "employeeNumber": NETSCAPE_LDAP + "3",
        "organizationName": X500ATTR_OID + "10",
        "eduPersonAssurance": EDUPERSON_OID + "11",
        "norEduOrgAcronym": NOREDUPERSON_OID + "6",
        "registeredAddress": X500ATTR_OID + "26",
        "physicalDeliveryOfficeName": X500ATTR_OID + "19",
        "associatedDomain": UCL_DIR_PILOT + "37",
        "l": X500ATTR_OID + "7",
        "stateOrProvinceName": X500ATTR_OID + "8",
        "federationFeideSchemaVersion": NOREDUPERSON_OID + "9",
        "pkcs9email": PKCS_9 + "1",
        "givenName": X500ATTR_OID + "42",
        "givenname": X500ATTR_OID + "42",
        "x500UniqueIdentifier": X500ATTR_OID + "45",
        "eduPersonNickname": EDUPERSON_OID + "2",
        "houseIdentifier": X500ATTR_OID + "51",
        "street": X500ATTR_OID + "9",
        "supportedAlgorithms": X500ATTR_OID + "52",
        "preferredLanguage": NETSCAPE_LDAP + "39",
        "postalAddress": X500ATTR_OID + "16",
        "email": PKCS_9 + "1",
        "norEduOrgUnitUniqueIdentifier": NOREDUPERSON_OID + "8",
        "eduPersonPrimaryOrgUnitDN": EDUPERSON_OID + "8",
        "c": X500ATTR_OID + "6",
        "teletexTerminalIdentifier": X500ATTR_OID + "22",
        "o": X500ATTR_OID + "10",
        "cACertificate": X500ATTR_OID + "37",
        "telexNumber": X500ATTR_OID + "21",
        "ou": X500ATTR_OID + "11",
        "initials": X500ATTR_OID + "43",
        "eduPersonOrgUnitDN": EDUPERSON_OID + "4",
        "deltaRevocationList": X500ATTR_OID + "53",
        "norEduPersonLIN": NOREDUPERSON_OID + "4",
        "supportedApplicationContext": X500ATTR_OID + "30",
        "eduPersonEntitlement": EDUPERSON_OID + "7",
        "generationQualifier": X500ATTR_OID + "44",
        "eduPersonAffiliation": EDUPERSON_OID + "1",
        "edupersonaffiliation": EDUPERSON_OID + "1",
        "eduPersonPrincipalName": EDUPERSON_OID + "6",
        "edupersonprincipalname": EDUPERSON_OID + "6",
        "localityName": X500ATTR_OID + "7",
        "owner": X500ATTR_OID + "32",
        "norEduOrgUnitUniqueNumber": NOREDUPERSON_OID + "2",
        "searchGuide": X500ATTR_OID + "14",
        "certificateRevocationList": X500ATTR_OID + "39",
        "organizationalUnitName": X500ATTR_OID + "11",
        "userCertificate": X500ATTR_OID + "36",
        "preferredDeliveryMethod": X500ATTR_OID + "28",
        "internationaliSDNNumber": X500ATTR_OID + "25",
        "uniqueMember": X500ATTR_OID + "50",
        "departmentNumber": NETSCAPE_LDAP + "2",
        "enhancedSearchGuide": X500ATTR_OID + "47",
        "userPKCS12": NETSCAPE_LDAP + "216",
        "eduPersonTargetedID": EDUPERSON_OID + "10",
        "norEduOrgUniqueNumber": NOREDUPERSON_OID + "1",
        "x121Address": X500ATTR_OID + "24",
        "destinationIndicator": X500ATTR_OID + "27",
        "eduPersonPrimaryAffiliation": EDUPERSON_OID + "5",
        "surname": X500ATTR_OID + "4",
        "jpegPhoto": UCL_DIR_PILOT + "60",
        "eduPersonScopedAffiliation": EDUPERSON_OID + "9",
        "edupersonscopedaffiliation": EDUPERSON_OID + "9",
        "protocolInformation": X500ATTR_OID + "48",
        "knowledgeInformation": X500ATTR_OID + "2",
        "employeeType": NETSCAPE_LDAP + "4",
        "userSMIMECertificate": NETSCAPE_LDAP + "40",
        "member": X500ATTR_OID + "31",
        "streetAddress": X500ATTR_OID + "9",
        "dmdName": X500ATTR_OID + "54",
        "postalCode": X500ATTR_OID + "17",
        "pseudonym": X500ATTR_OID + "65",
        "dnQualifier": X500ATTR_OID + "46",
        "crossCertificatePair": X500ATTR_OID + "40",
        "eduPersonOrgDN": EDUPERSON_OID + "3",
        "authorityRevocationList": X500ATTR_OID + "38",
        "displayName": NETSCAPE_LDAP + "241",
        "businessCategory": X500ATTR_OID + "15",
        "serialNumber": X500ATTR_OID + "5",
        "norEduOrgUniqueIdentifier": NOREDUPERSON_OID + "7",
        "st": X500ATTR_OID + "8",
        "carLicense": NETSCAPE_LDAP + "1",
        "presentationAddress": X500ATTR_OID + "29",
        "sn": X500ATTR_OID + "4",
        "cn": X500ATTR_OID + "3",
        "domainComponent": UCL_DIR_PILOT + "25",
        "labeledURI": UMICH + "57",
        "uid": UCL_DIR_PILOT + "1",
        "schacMotherTongue": SCHAC + "1",
        "schacGender": SCHAC + "2",
        "schacDateOfBirth": SCHAC + "3",
        "schacPlaceOfBirth": SCHAC + "4",
        "schacCountryOfCitizenship": SCHAC + "5",
        "schacSn1": SCHAC + "6",
        "schacSn2": SCHAC + "7",
        "schacPersonalTitle": SCHAC + "8",
        "schacHomeOrganization": SCHAC + "9",
        "schacHomeOrganizationType": SCHAC + "10",
        "schacCountryOfResidence": SCHAC + "11",
        "schacUserPresenceID": SCHAC + "12",
        "schacPersonalPosition": SCHAC + "13",
        "schacPersonalUniqueCode": SCHAC + "14",
        "schacPersonalUniqueID": SCHAC + "15",
        "schacExpiryDate": SCHAC + "17",
        "schacUserPrivateAttribute": SCHAC + "18",
        "schacUserStatus": SCHAC + "19",
        "schacProjectMembership": SCHAC + "20",
        "schacProjectSpecificRole": SCHAC + "21",
    },
}



# Copyright (C) 2012 Yaco Sistemas (http://www.yaco.es)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import base64
import logging
import re
import urllib
import zlib
from typing import Optional

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import resolve_url
from django.urls import NoReverseMatch
from django.utils.http import url_has_allowed_host_and_scheme

from saml2.config import SPConfig
from saml2.mdstore import MetaDataMDX
from saml2.s_utils import UnknownSystemEntity

logger = logging.getLogger(__name__)


def get_custom_setting(name: str, default=None):
    return getattr(settings, name, default)


def available_idps(config: SPConfig, langpref=None, idp_to_check: str = None) -> dict:
    if langpref is None:
        langpref = "en"

    idps = set()

    for metadata in config.metadata.metadata.values():
        # initiate a fetch to the selected idp when using MDQ, otherwise the MetaDataMDX is an empty database
        if isinstance(metadata, MetaDataMDX) and idp_to_check:
            m = metadata[idp_to_check]
        result = metadata.any("idpsso_descriptor", "single_sign_on_service")
        if result:
            idps.update(result.keys())

    return {idp: config.metadata.name(idp, langpref) for idp in idps}


def get_idp_sso_supported_bindings(
    idp_entity_id: Optional[str] = None, config: Optional[SPConfig] = None
) -> list:
    """Returns the list of bindings supported by an IDP
    This is not clear in the pysaml2 code, so wrapping it in a util"""
    if config is None:
        # avoid circular import
        from .conf import get_config

        config = get_config()
    # load metadata store from config
    meta = getattr(config, "metadata", {})
    # if idp is None, assume only one exists so just use that
    if idp_entity_id is None:
        try:
            idp_entity_id = list(available_idps(config).keys())[0]
        except IndexError:
            raise ImproperlyConfigured("No IdP configured!")
    try:
        return list(
            meta.service(
                idp_entity_id, "idpsso_descriptor", "single_sign_on_service"
            ).keys()
        )
    except UnknownSystemEntity:
        raise UnknownSystemEntity
    except Exception as e:
        logger.error(f"get_idp_sso_supported_bindings failed with: {e}")


def get_location(http_info):
    """Extract the redirect URL from a pysaml2 http_info object"""
    try:
        headers = dict(http_info["headers"])
        return headers["Location"]
    except KeyError:
        return http_info["url"]


def get_fallback_login_redirect_url():
    login_redirect_url = get_custom_setting("LOGIN_REDIRECT_URL", "/")
    return resolve_url(
        get_custom_setting("ACS_DEFAULT_REDIRECT_URL", login_redirect_url)
    )


def validate_referral_url(request, url):
    # Ensure the url is even a valid URL; sometimes the given url is a
    # RelayState containing PySAML data.
    # Some technically-valid urls will be fail this check, so the
    # SAML_STRICT_URL_VALIDATION setting can be used to turn off this check.
    # This should only happen if there is no slash, host and/or protocol in the
    # given URL. A better fix would be to add those to the RelayState.
    saml_strict_url_validation = getattr(
        settings,
        "SAML_STRICT_URL_VALIDATION",
        True
    )
    try:
        if saml_strict_url_validation:
            # This will also resolve Django URL pattern names
            url = resolve_url(url)
    except NoReverseMatch:
        logger.debug("Could not validate given referral url is a valid URL")
        return None

    # Ensure the user-originating redirection url is safe.
    # By setting SAML_ALLOWED_HOSTS in settings.py the user may provide a list of "allowed"
    # hostnames for post-login redirects, much like one would specify ALLOWED_HOSTS .
    # If this setting is absent, the default is to use the hostname that was used for the current
    # request.
    saml_allowed_hosts = set(
        getattr(settings, "SAML_ALLOWED_HOSTS", [request.get_host()])
    )

    if not url_has_allowed_host_and_scheme(url=url, allowed_hosts=saml_allowed_hosts):
        logger.debug("Referral URL not in SAML_ALLOWED_HOSTS or of the origin "
                     "host.")
        return None

    return url


def saml2_from_httpredirect_request(url):
    urlquery = urllib.parse.urlparse(url).query
    b64_inflated_saml2req = urllib.parse.parse_qs(urlquery)["SAMLRequest"][0]

    inflated_saml2req = base64.b64decode(b64_inflated_saml2req)
    deflated_saml2req = zlib.decompress(inflated_saml2req, -15)
    return deflated_saml2req


def get_session_id_from_saml2(saml2_xml):
    saml2_xml = saml2_xml.decode() if isinstance(saml2_xml, bytes) else saml2_xml
    return re.findall(r'ID="([a-z0-9\-]*)"', saml2_xml, re.I)[0]


def get_subject_id_from_saml2(saml2_xml):
    saml2_xml = saml2_xml if isinstance(saml2_xml, str) else saml2_xml.decode()
    re.findall('">([a-z0-9]+)</saml:NameID>', saml2_xml)[0]


def add_param_in_url(url: str, param_key: str, param_value: str):
    params = list(url.split("?"))
    params.append(f"{param_key}={param_value}")
    new_url = params[0] + "?" + "".join(params[1:])
    return new_url


def add_idp_hinting(request, http_response) -> bool:
    idphin_param = getattr(settings, "SAML2_IDPHINT_PARAM", "idphint")
    urllib.parse.urlencode(request.GET)

    if idphin_param not in request.GET.keys():
        return False

    idphint = request.GET[idphin_param]
    # validation : TODO -> improve!
    if idphint[0:4] != "http":
        logger.warning(
            f'Idp hinting: "{idphint}" doesn\'t contain a valid value.'
            "idphint paramenter ignored."
        )
        return False

    if http_response.status_code in (302, 303):
        # redirect binding
        # urlp = urllib.parse.urlparse(http_response.url)
        new_url = add_param_in_url(http_response.url, idphin_param, idphint)
        return HttpResponseRedirect(new_url)

    elif http_response.status_code == 200:
        # post binding
        res = re.search(
            r'action="(?P<url>[a-z0-9\:\/\_\-\.]*)"',
            http_response.content.decode(),
            re.I,
        )
        if not res:
            return False
        orig_url = res.groupdict()["url"]
        #
        new_url = add_param_in_url(orig_url, idphin_param, idphint)
        content = http_response.content.decode().replace(orig_url, new_url).encode()
        return HttpResponse(content)

    else:
        logger.warning(
            f"Idp hinting: cannot detect request type [{http_response.status_code}]"
        )
    return False



# Copyright (C) 2011-2012 Yaco Sistemas <lorenzo.gil.sanchez@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import codecs
import os

from setuptools import find_packages, setup


def read(*rnames):
    return codecs.open(
        os.path.join(os.path.dirname(__file__), *rnames), encoding="utf-8"
    ).read()


setup(
    name="djangosaml2",
    version="1.9.0",
    description="pysaml2 integration for Django",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.1",
        "Framework :: Django :: 4.2",
        "Framework :: Django :: 5.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Security",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
    ],
    keywords="django,pysaml2,sso,saml2,federated authentication,authentication",
    author="Yaco Sistemas and independent contributors",
    author_email="lorenzo.gil.sanchez@gmail.com",
    maintainer="Giuseppe De Marco",
    url="https://github.com/IdentityPython/djangosaml2",
    download_url="https://pypi.org/project/djangosaml2/",
    license="Apache 2.0",
    packages=find_packages(exclude=["tests", "tests.*"]),
    include_package_data=True,
    zip_safe=False,
    install_requires=["defusedxml>=0.4.1", "Django>=3.2", "pysaml2>=6.5.1"],
)
